# -*- coding: utf-8 -*-
from openerp import models, fields, api, exceptions, _, SUPERUSER_ID
import logging

_logger = logging.getLogger(__name__)


class plan_generator(models.TransientModel):
    _name = 'plan.generator'
    _description = 'Generates or delete a financing plan'

    financing_config_ids = fields.Many2many('financing.config', string='Financing configs')
    product_ids = fields.Many2many('product.product', string='Products')
    category_ids = fields.Many2many('product.category', string='Categories')
    pos_config_ids = fields.Many2many('pos.config', string='PoS Configuration')
    percent_pu = fields.Float('Percentage to apply for product PU')
    financing_total_rate = fields.Float('Financing Total Rate')
    use_financing_total_rate = fields.Boolean('Use Custom Rate')
    warn_accepted = fields.Boolean('Is warning accepted?')

    @api.onchange('category_ids')
    def onchange_category_ids(self):
        category_ids = self.category_ids.ids
        if category_ids:
            self._cr.execute("SELECT DISTINCT pp.id FROM product_product pp JOIN product_template pt ON pt.id = pp.product_tmpl_id WHERE pt.categ_id IN %s;", (tuple(category_ids), ))
            prod_ids = [i for i, in self._cr.fetchall()]
            if prod_ids:
                new_products = self.env['product.product'].browse(prod_ids)
                actual_products_ids = self.product_ids
                all_product_ids = new_products + actual_products_ids
                self.product_ids = [(6, 0, all_product_ids.ids)]

    @api.multi
    def generate(self):
        if not any(self.pos_config_ids):
            raise exceptions.ValidationError(_('You must select at least one pos configuration'))
        if not any(self.financing_config_ids):
            raise exceptions.ValidationError(_('You must select at least one financing configuration'))
        if not any(self.product_ids):
            raise exceptions.ValidationError(_('You must at least select one product'))
        pf_obj = self.env['product.financing']
        for product in self.product_ids:
            for financing in self.financing_config_ids:
                for pos_config in self.pos_config_ids:
                    self._cr.execute("SELECT id FROM product_financing WHERE product_id=%s AND financing_config_id=%s AND pos_config_id=%s", (product.id, financing.id, pos_config.id))
                    if self._cr.rowcount:
                        _logger.warning('The financing for product #%s, financing #%s and pos #%s is already configured. Omit.')
                        continue
                    # Set up financing
                    pf_vals = {
                        'product_id': product.id,
                        'financing_config_id': financing.id,
                        'pos_config_id': pos_config.id,
                        'customer_type': financing.customer_type,
                    }
                    pf_created = pf_obj.create(pf_vals)
                    pf_created.onchange_financing()

    @api.multi
    def remove(self):
        if self.env.user.id != SUPERUSER_ID:
            raise exceptions.ValidationError(_('This is a REALLY dangerous action, thus only the admin can perform it!'))
        self._cr.execute("DELETE FROM product_financing WHERE product_id IN %s AND financing_config_id IN %s AND pos_config_id IN %s;", (tuple(self.product_ids.ids), tuple(self.financing_config_ids.ids), tuple(self.pos_config_ids.ids)))


    @api.multi
    def update(self):
        if not any(self.product_ids):
            raise exceptions.ValidationError(_('You must at least select one product'))
        if not any(self.pos_config_ids):
            raise exceptions.ValidationError(_('You must select at least one pos configuration'))
        if not any(self.financing_config_ids):
            raise exceptions.ValidationError(_('You must select at least one financing configuration'))
        if (len(self.financing_config_ids) > 1) and (self.use_financing_total_rate):
            if not self.warn_accepted:
                self.warn_accepted = True
                self._cr.commit()
                raise exceptions.ValidationError(_('You have several financings selected, and this will override the total_rate of selected financings to %s. If you wish to continue, discard this message and update again') % self.financing_total_rate)
        prod_pu_percent = self.percent_pu
        if not (-100 <= prod_pu_percent <= 100):
            raise exceptions.ValidationError(_('Invalid percentage for product'))
        else:
            rate_to_apply = prod_pu_percent / float(100)
        pf_to_update_lst = []
        for product in self.product_ids:
            product_actual_price = product.lst_price
            new_price_unit = product_actual_price + (product_actual_price * rate_to_apply)
            for financing in self.financing_config_ids:
                if self.use_financing_total_rate:
                    financing_total_rate = self.financing_total_rate
                else:
                    financing_total_rate = financing.total_rate

                for pos_config in self.pos_config_ids:
                    self._cr.execute("SELECT pf.id, fc.quota_qty, pf.product_id FROM product_financing pf JOIN financing_config fc ON fc.id=pf.financing_config_id WHERE pf.product_id=%s AND pf.financing_config_id=%s AND pf.pos_config_id=%s", (product.id, financing.id, pos_config.id))
                    if self._cr.rowcount:  # TODO [ssaid@02/11/2017]: Here we should override the configuration
                        pf_id, quota_qty, prod_prod_id = self._cr.fetchone()
                        # pf = pf_obj.browse(fc_id)
                        quota_amount = (new_price_unit * (1 + financing_total_rate)) / quota_qty
                        pf_to_update_lst.append((pf_id, prod_prod_id, quota_amount))
                        _logger.info('Appended to update product.financing for product #%s, financing #%s and pos #%s.' % (product.id, financing.id, pos_config.id))
                    else:  # No product.financing configured
                        _logger.warning('The product.financing for product #%s, financing #%s and pos #%s it will NOT be generated.' % (product.id, financing.id, pos_config.id))
                        continue
                        # pf_vals = {
                        #     'product_id': product.id,
                        #     'financing_config_id': financing.id,
                        #     'pos_config_id': pos_config.id,
                        #     'customer_type': financing.customer_type,
                        # }
                        # pf_created = pf_obj.create(pf_vals)
                        # pf_created.quota_amount = (new_price_unit * (1 + financing_total_rate)) / financing.quota_qty

        # Flight ready, now trigger update
        if pf_to_update_lst:
            product_ids = list(set([i[1] for i in pf_to_update_lst]))
            for pf in pf_to_update_lst:
                _logger.info('Write in db: %s', pf)
                self._cr.execute("UPDATE product_financing SET quota_amount=%(quota_amount)s WHERE id=%(pf_id)s", {'quota_amount': pf[2], 'pf_id': pf[0]})
            _logger.info('Finished write to db. Next step, compute...')
            prod_prod_obj = self.env['product.product']
            products = prod_prod_obj.browse(product_ids)
            products.compute_financing_tables()

